
import pandas as pd
from jumps.Jump_Tunnel_HIVE import Jump_Tunnel_HIVE
from jumps.SSH_Tunnel import  SSH_Tunnel
from jumps.hive_client import hive_client

username = 'zhuzheng'
password = "abc123."
localhost="127.0.0.1"
localbindport=3560
cli = hive_client(localhost, username, password, localbindport)
conn = cli.connhive()
table = 'tab_client_label'
partions_param_dict = {'client_nmbr': 'AA74', 'batch': 'p1'}
query_fileds_list = ['gid', 'realname', 'card']
querylimit = 1000
df = cli.query_selectFileds_Dataframe(conn, table, query_fileds_list, partions_param_dict, querylimit)
print(df.shape)

def gethive(localport):
  jumphost = '117.48.195.186'
  jumpport = 2222
  jumpuser = 'dm'
  jumppwd = 'Vts^pztbvE339@Rw'
  tunnelhost = '172.16.16.32'
  tunnelhiveport = 10000
  localhost = '127.0.0.1'
  localbindport =localport
  username = 'zhuzheng'
  password = "abc123."
  jump = Jump_Tunnel_HIVE(jumphost, jumpport, jumpuser, jumppwd, tunnelhost, tunnelhiveport, localhost, localbindport,
    username, password)
  return jump

def demo1():
    table = 'tab_client_label'
    partions_param_dict = {'client_nmbr': 'AA75', 'batch': 'p1'}
    query_fileds_list = ['gid', 'realname', 'card']
    querylimit = 1000
    jump=gethive(4100)
    df2=jump.get_JumpTunnel_df(table,partions_param_dict,query_fileds_list,querylimit)
    return df2
def demo2():
  table = 'tab_client_label'
  partions_param_dict = {'client_nmbr': 'AA75', 'batch': 'p1'}
  jump =gethive(4322)
  df2 = jump.get_JumpTunnel_table_partitions_df(table,partions_param_dict,1000)
  return df2

def demo3():
  jump = gethive(4534)
  hsql="select * from fkdb.tab_client_label where  client_nmbr= 'AA75' and batch= 'p1' limit 500"
  df2=jump.get_JumpTunnel_hsql_df(hsql)
  return df2

df3=demo1()
print(df3.shape)
print(df3.columns)
print(df3.head(100))
# df3=demo2()
# print(df3.shape)
# print(df3.columns)
# print(df3.head(100))
# df3=demo3()
# print(df3.shape)
# print(df3.columns)
# print(df3.head(100))


from pyhive import hive
import pandas as  pd
import paramiko
import logging
import selectors
import socketserver
import threading
import time
from jumps.hive_client import hive_client
from jumps.SSH_Tunnel import  SSH_Tunnel

logger = logging.getLogger(
    name=__name__,
)

class Jump_Tunnel_HIVE:
  logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
  logger = logging.getLogger(__name__)
  def __init__(self,jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelAPPport,localhost,localbindport,hiveusername, hivepassword):
    self.jumphost=jumphost.strip()
    self.jumpport=jumpport
    self.jumpuser=jumpuser.strip()
    self.jumppwd=jumppwd.strip()
    self.tunnelhost=tunnelhost.strip()
    self.tunnelAPPport=tunnelAPPport
    self.localhost=localhost.strip()
    self.localBindport=localbindport
    self.hiveusername=hiveusername.strip()
    self.hivepassword=hivepassword.strip()
  def  jump_connect(self):
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(
      paramiko.AutoAddPolicy(),
    )
    client.connect(
      hostname=self.jumphost,
      port=self.jumpport,
      username=self.jumpuser,
      password=self.jumppwd,
    )
    logger.info(msg="jump server info :{host: %s, port: %d,username: %s,password: %s}"%(self.jumphost,self.jumpport,self.jumpuser,self.jumppwd))

    return client
  def  jump_con_tunnel(self):
    client=self.jump_connect()
    tunnel=SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost,self.localBindport),
    )
    logger.info(msg="the ssh tunnel  for jump server %s" %tunnel)
    return tunnel
  # 获取hive数据生成 pandas dataframe ，
  # 参数 ：表名  分区参数字典  要查询的字段名列表，查询数量 默认  ，查询数据库 默认  登录hive方式 默认
  # pip install paramiko paramiko_tunnel  pyhive
  def get_JUMP_df(self,table, partions_param_dict, query_fileds_list, query_limit=100, database='fkdb', auth='LDAP'):
    tun=self.jump_con_tunnel()
    with tun:
       hive_cli = hive_client(self.localhost, self.hiveusername,self.hivepassword,self.localBindport)
       conn = hive_cli.connhive(database, auth)
       pd_dataframe = hive_cli.query_selectFileds_Dataframe(conn, table, query_fileds_list, partions_param_dict, query_limit)
       return pd_dataframe


  def get_JumpTunnel_df(self,table, partions_param_dict, query_fileds_list, query_limit=100, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
      paramiko_session=client,
      remote_host=self.tunnelhost,
      remote_port=self.tunnelAPPport,
      bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel :
       logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
       hive_cli = hive_client(self.localhost, self.hiveusername,self.hivepassword,self.localBindport)
       conn = hive_cli.connhive(database, auth)
       pd_dataframe = hive_cli.query_selectFileds_Dataframe(conn, table, query_fileds_list, partions_param_dict, query_limit)
       return pd_dataframe
  def get_JumpTunnel_hsql_df(self,hsql, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      pd_dataframe=hive_cli.queryby_hsql_df(conn,hsql)
      return  pd_dataframe

  def get_JumpTunnel_table_partitions_df(self,table,param_dict,limit=None, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      pd_dataframe = hive_cli.query_AllFileds_Dataframe(conn,table,param_dict,limit)
      return pd_dataframe

  def alter_table_Tunnel_by_hsql(self,hsql, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      cursor=conn.cursor()
      try:
        cursor.execute(hsql)
        logger.info(msg="hive cursor exec  successfully for : [ %s ]"%hsql)
      except:
        logger.error(msg="hive cursor  exec failed for [ %s ]" % hsql)


import sys
from PyQt5.QtGui import QPixmap,QPalette
from PyQt5 import QtCore,QtWidgets
from PyQt5.QtWidgets import QWidget,QInputDialog,QMainWindow,QDialog,QLabel,QLineEdit,QGridLayout,  QToolTip,QPushButton, QApplication
from jumps.Jump_Tunnel import Jump_Tunnel
from PyQt5.QtCore import Qt
import click
import  time
import  logging
import threading

jumphost = '117.48.195.186'
jumpport = 2222
jumpuser = 'dm'
jumppwd = 'Vts^pztbvE339@Rw'
tunnelhost = '172.16.16.32'
tunnelappport = 10000
localhost = '127.0.0.1'
localbindport = 4800
daemonsecond=2000
logger = logging.getLogger('ssh-jump-hive-gui')
class JumpTunnel(QWidget):

  def __init__(self):
    super().__init__()

    self.my_UI()


  def my_UI(self):


    jhLabel=QLabel("JumpHost:")
    jpLabel=QLabel("JumpPort:")
    juLable=QLabel("JumpUser:")
    jpwdLabel=QLabel("JumpPwd:")
    thLabel=QLabel("TunnelHost:")
    tpLabel=QLabel("TunnelPort:")
    lhLabel=QLabel("LocalHost:")
    lpLabel=QLabel("LocalPort:")
    gitLabel = QLabel("GithubRepo:")
    dtLabel=QLabel("DaemonSecond:")

    self.jumpHost=QLineEdit("117.48.195.186")
    self.jumpPort=QLineEdit("2222")
    self.jumpUser=QLineEdit("dm")
    self.jumpPwd=QLineEdit("Vts^pztbvE339@Rw")
    self.tunnelHost=QLineEdit("172.16.16.32")
    self.tunnelPort=QLineEdit("10000")
    self.localHost=QLineEdit("127.0.0.1")
    self.localPort=QLineEdit("3560")
    self.daemonSecond=QLineEdit("21600")
    github=QLineEdit("https://github.com/mullerhai/sshjumphive")
    self.btnConn = QPushButton("Trun ON", self)
    self.btnClose=QPushButton("Trun Off",self)
    self.grid=QGridLayout()
    self.grid.setSpacing(10)
    self.grid.addWidget(jhLabel,2,0)
    self.grid.addWidget(self.jumpHost,2,1)
    self.grid.addWidget(jpLabel,2,2)
    self.grid.addWidget(self.jumpPort,2,3)
    self.grid.addWidget(juLable,3,0)
    self.grid.addWidget(self.jumpUser,3,1)
    self.grid.addWidget(jpwdLabel,3,2)
    self.grid.addWidget(self.jumpPwd,3,3)
    self.grid.addWidget(thLabel,5,0)
    self.grid.addWidget(self.tunnelHost,5,1)
    self.grid.addWidget(tpLabel,5,2)
    self.grid.addWidget(self.tunnelPort,5,3)
    self.grid.addWidget(lhLabel,7,0)
    self.grid.addWidget(self.localHost,7,1)
    self.grid.addWidget(lpLabel,7,2)
    self.grid.addWidget(self.localPort,7,3)
    self.grid.addWidget(gitLabel,8,0)
    self.grid.addWidget(github,8,1)
    self.grid.addWidget(dtLabel,8,2)
    self.grid.addWidget(self.daemonSecond,8,3)
    self.grid.addWidget(self.btnConn,9,0)
    self.grid.addWidget(self.btnClose,9,3)
    pixmap = QPixmap("../img/guilogo.jpg")
    pixmap=pixmap.scaledToHeight(80)
    pixmap=pixmap.scaledToWidth(180)
    lbl = QLabel(self)
    lbl.setFixedHeight(80)
    lbl.setFixedWidth(180)
    lbl.setPixmap(pixmap)
    self.grid.addWidget(lbl,10,1)
    pixfox = QPixmap("../img/tunnel.jpg")
    pixfox=pixfox.scaledToHeight(90)
    pixfox=pixfox.scaledToWidth(90)
    lblfox = QLabel(self)
    lblfox.setFixedHeight(90)
    lblfox.setFixedWidth(90)
    lblfox.setPixmap(pixfox)
    self.grid.addWidget(lblfox,10,2)
    self.btnConn.clicked.connect(self.buttonClicked)
    self.btnClose.clicked.connect(self.btnCloseSession)
    self.setLayout(self.grid)
    self.setWindowTitle('SSH-Jump-Hive')
    self.setGeometry(300, 300, 490, 450)
    self.show()

  def btnCloseSession(self):
    # text, ok = QInputDialog.getText(self, 'Turn Off',
    #   'Please Input 1 then Trun off tunnel :')
    # logging.warn(msg="Will kill recently ssh tunnle process")
    # if ok and text=='1':
      try:
        self.jump_tunnel.client.close()
        logging.info(msg="ssh_tunnel turn off  successfully")
        sucTLabel = QLabel("turn off Success")
        self.grid.addWidget(sucTLabel, 9, 1)
        # text, ok = QInputDialog.getText(self, 'Success',
        #   'ssh_tunnel turn off  successfully close dialog ok')
      except:
        failedTLabel = QLabel("turn off be Failed")
        self.grid.addWidget(failedTLabel, 9, 2)
        # text, ok = QInputDialog.getText(self, 'Failed',
        #   'ssh_tunnel turn off  failed check the config')
        logging.error(msg="ssh_tunnel turn off failed,please try again")
    # else:
    #   failedTLabel = QLabel("turn off Failed")
    #   self.grid.addWidget(failedTLabel, 9, 2)
      # text, ok = QInputDialog.getText(self, 'Failed',
      #   'ssh_tunnel turn off  failed check the config')
  def buttonClicked(self):  # 在buttonClikced()方法中，我们调用sender()方法来判断哪一个按钮是我们按下的
    jumphost=self.jumpHost.text().strip()
    jumpuser=self.jumpUser.text().strip()
    jumppwd=self.jumpPwd.text().strip()
    tunnelhost=self.tunnelHost.text().strip()
    localhost=self.localHost.text().strip()

    logging.info(msg=self.jumpHost.text()+"%%"+self.jumpUser.text()+"%%"+self.jumpPwd.text())
    try:
      jumpport = (int(self.jumpPort.text().strip()) if self.jumpPort.text().strip() != None else 2222)
      tunnelappport = (int(self.tunnelPort.text().strip()) if self.tunnelPort.text().strip() != None else 10000)
      localbindport = (int(self.localPort.text().strip()) if self.localPort.text().strip() != None else 4320)
      daemonsecond = (int(self.daemonSecond.text().strip()) if self.daemonSecond.text().strip() != None else 21600)

      self.jump_tunnel=Jump_Tunnel(jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelappport,localhost,localbindport)
      tunnel_conn=self.jump_tunnel.jump_con_tunnel()

      with  tunnel_conn:
        #time.sleep(0.1)
        logging.info(msg="启动成功")
        sucLabel = QLabel("Connect Success")
        self.grid.addWidget(sucLabel, 9, 2)
        # pe = QPalette()
        # pe.setColor(QPalette.WindowText, Qt.red)
        #sucLabel.setAutoFillBackground(pe)

        text, ok = QInputDialog.getText(self, 'Success',
          'connect ssh tunnel successfully close dialog ok')
        time.sleep(daemonsecond)
    except:
      logging.info(msg="启动失败")
      failedLabel=QLabel("Connect Failed")
      self.grid.addWidget(sucLabel, 9, 2)
      # text, ok = QInputDialog.getText(self, 'Failed',
      #   'connect ssh tunnel failed check the config')

    #sender = self.sender()
    # self.showMessage(sender.text() + ' 是发送者')

def main():
  app = QApplication(sys.argv)
  jtGui = JumpTunnel()
  sys.exit(app.exec_())


if __name__ == '__main__':
  app = QApplication(sys.argv)
  ex = JumpTunnel()
  sys.exit(app.exec_())



from ftplib import FTP
from ftplib import FTP_TLS

class  ftps_client:
  ##初始化的时候会把登录参数赋值初始化
  def __init__(self,host,user,pwd,port=21):
    self.host=host
    self.port=port
    self.user=user
    self.pwd=pwd
    #self._old_makepasv=FTP_TLS.makepasv

## ftp 登录项  含有闭包项
  def login(self,debug=2,set_pasv=True):
    _old_makepasv = FTP_TLS.makepasv
    def _new_makepasv(self):
      host, port = _old_makepasv(self)
      host = self.sock.getpeername()[0]
      return host, port
    FTP_TLS.makepasv = _new_makepasv
    ftps = FTP_TLS(self.host)
    ftps.set_debuglevel(debug)
    ftps.auth()
    ftps.login(self.user,self.pwd)
    ftps.makepasv()
    ftps.sendcmd('pbsz 0')
    ftps.set_pasv(set_pasv)
    ftps.prot_p()
    print("hello ")
    ftps.getwelcome()
    return ftps

  #显示  目录下的 文件列表
  def ftplistDir(self,ftps,sever_path):
    ftps.cwd("/")#首先切换得到根目录下，否则会出现问题
    ftps.cwd(sever_path)
    files = ftps.nlst()
    for f in files:
      print(f)

# 下载服务器文件
  def  ftpDownloadSeverFile(self,ftps,sever_path,sever_file,new_localfile,buffersize=1024):
    ftps.cwd("/")
    ftps.cwd(sever_path)
    with open(new_localfile , 'wb')as download_file:
      ftps.retrbinary('RETR %s' %sever_file , download_file.write, buffersize)

##上传文件
  def  ftpUploadLocalFile(self,ftps,local_filepath,sever_path,new_severfile,buffersize=1024):
    ftps.cwd("/")
    ftps.cwd(sever_path)
    with open(local_filepath,'rb') as  upload_file:
      ftps.storbinary('STOR ' + new_severfile, upload_file, buffersize)


##测试使用 通过
if __name__ == '__main__':
  host = 'ftps.baidu.com'
  port = '21'
  user = 'zh****eng'
  pwd = 'zz****mt.2'
  ip = '117.35.45.150'

  cli=ftp_client(host,user,pwd,port)
  fs= cli.login(2,True)
  #fs.makepasv()
  # files = []
  # files = fs.nlst()
  # for f in files:
  #   print(f)
  path='china'
  cli.ftplistDir(fs,path)


import paramiko
import logging
import selectors
import socketserver
import threading
import time
from .SSH_Tunnel import SSH_Tunnel

logger = logging.getLogger(
    name=__name__,
)

class Jump_Tunnel:

  def __init__(self,jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelAPPport,localhost,localbindport):
    self.jumphost=jumphost.strip()
    self.jumpport=jumpport
    self.jumpuser=jumpuser.strip()
    self.jumppwd=jumppwd.strip()
    self.tunnelhost=tunnelhost
    self.tunnelAPPport=tunnelAPPport
    self.localhost=localhost.strip()
    self.localBindport=localbindport

  def  jump_connect(self):
    self.client = paramiko.SSHClient()
    self.client.set_missing_host_key_policy(
      paramiko.AutoAddPolicy(),
    )
    self.client.connect(
      hostname=self.jumphost,
      port=self.jumpport,
      username=self.jumpuser,
      password=self.jumppwd,
    )
    logger.info(msg="jump server info :{host: %s, port: %d,username: %s,password: %s}"%(self.jumphost,self.jumpport,self.jumpuser,self.jumppwd))

    return self.client
  def  jump_con_tunnel(self):
    client=self.jump_connect()
    tunnel=SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost,self.localBindport),
    )
    logger.info(msg="the ssh tunnel  for jump server %s" %tunnel)
    return tunnel
  def jump_break_conn_tunnel(self):
    self.client.close()
    logging.warn(msg="ssh_tunnel have been turn off")
  def  hold_ssh_tunnel_session(self,daemonSecond):
    jump_tunnel=Jump_Tunnel(self.jumphost,self.jumpport,self.jumpuser,self.jumppwd,self.tunnelhost,self.tunnelAPPport,self.localhost,self.localbindport)
    tunnel=jump_tunnel.jump_con_tunnel()
    try:
      with  tunnel:
        logger.info("ssh_tunnel_session create successfully ")
        time.sleep(daemonSecond)
    except:
      logger.error("ssh_tunnel_session create failed")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
ssh-jump-hive 主程序
"""
# import argparse
# import gettext
# import io
# import json
import logging
import sys
from jumps.Jump_Tunnel import Jump_Tunnel
import click
import  time


APP_DESC = """
                                                                                                    
     ####   ####  #    #                  # #    # #    # #####              #    # # #    # ###### 
    #      #      #    #                  # #    # ##  ## #    #             #    # # #    # #      
     ####   ####  ######    #####         # #    # # ## # #    #    #####    ###### # #    # #####  
         #      # #    #                  # #    # #    # #####              #    # # #    # #      
    #    # #    # #    #             #    # #    # #    # #                  #    # #  #  #  #      
     ####   ####  #    #              ####   ####  #    # #                  #    # #   ##   ###### 

                        ---- A Terminal Tools For Jump Gateway Server
                        ---- For Hive HDFS FTP FTPS Mysql Neo4j SPARK 
                        ---- Redis MongoDB Hadoop Zookeeper Mpi Tomcat 

    @author Muller Helen (https://github.com/mullerhai/sshjumphive)
                                    last_update 2018-04-27 11:33:59
"""

logger = logging.getLogger('ssh-jump-hive-shell')
Process_HoldOn_DaemonSecond=21600 #六小时  six hours

def check_setting_and_env():
    logger.info("程序正在启动,检查环境配置")
    if sys.version_info < (3, 2):
        raise RuntimeError("at least Python 3.3 is required!!")
    logger.info("开始配置环境")



@click.command()
#@click.argument('jhost',default="117.48.195.186", required=True)
@click.option('-jh', '--jumphost', default="117.48.195.186", help='Jump Gateway Server host 跳板机ssh 主机名, 默认117.48.195.186')
@click.option('-jp', '--jumpport', default=2222, help='Jump Gateway Server port跳板机ssh登录端口号, 默认2222')
@click.option('-ju', '--jumpuser', default='dm', help='Jump Gateway Server login user 跳板机 ssh登录用户名')
@click.option('-jpd', '--jumppwd', default="Vts^pztbvE339@Rw",  help='Jump Gateway Server login user password 跳板机登录用户密码')
@click.option('-th', '--tunnelhost', default='172.16.16.32', help='ssh-tunnel 隧道 host ')
@click.option('-tp', '--tunnelappport', default=10000, help='ssh-tunnel Application port隧道 目标程序的端口号 默认为 hive 10000 ')
@click.option('-lh', '--localhost', default='127.0.0.1', help='localhost本机 host ,默认127.0.0.1 ')
@click.option('-lp', '--localbindport', default="4230", help='localbindport 本机 被绑定的端口号')
@click.option('-dt', '--daemonsecond', default="21600", help='ssh_tunnel_daemon_session_hold_on_second six hours, ssh 隧道 后台线程 保持时间 默认为六小时')
def parse_command(jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelappport,localhost,localbindport,daemonsecond):
    logger.info("正在检查环境")
    check_setting_and_env()
    if jumphost=="117.48.195.186":
      logger.info(
        msg="jumphost: %s, jumpport: %s,  tunnelhost: %s, tunnelAppPort: %s, localhost: %s, localBindport: %s," % (
        jumphost, jumpport, tunnelhost, tunnelappport, localhost, localbindport))
    else:
      logger.info(
        msg="jumphost: %s, jumpport: %s, jumpuser: %s, jumppwd: %s, tunnelhost: %s, tunnelAppPort: %s, localhost: %s, localBindport: %s,"%(
          jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelappport,localhost,localbindport))

    logger.info("环境检查完毕,正在开启 start ssh_tunnel for you(请等待15s~30s)")
    start_jump_tunnel_client(jumphost.strip(),int(jumpport.strip()),jumpuser.strip(),jumppwd.strip(),tunnelhost.strip(),int(tunnelappport.strip()),localhost.strip(),int(localbindport.strip()),int(daemonsecond.strip()))


def start_jump_tunnel_client(jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelAPPport,localhost,localbindport,daemonsecond):
    jump_tunnel=Jump_Tunnel(jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelAPPport,localhost,localbindport)
    tunnel=jump_tunnel.jump_con_tunnel()
    try:
      with  tunnel:
        #jump_tunnel.jump_con_tunnel().start()
        logger.info("ssh tunnel start successfully")
        logger.info("please  hold on the shell ,do not turn off it ! or  you can turn it like daemon process")
        logger.info("please make sure the  ssh_tunnel daemon_session default hold on six hours , ")
        logger.info("if you want to change it   please set the config param -dt")
        logger.info("if you want to close the ssh_tunnel_session just close the shell !")
        time.sleep(daemonsecond)
    except:
      logger.error("ssh tunnel start failed,please check the config param or maybe localport has been used  change it!")

def main():
    print(APP_DESC)
    time.sleep(0.02)
    parse_command()



if __name__ == "__main__":
    main()



from pyhive import hive
import pandas as  pd
import logging




class hive_client:

  def __init__(self,host,user,pwd,port=10000,database='fkdb',auth='LDAP'):
    self.host=host
    self.user=user
    self.pwd=pwd
    self.port=port
    self.database=database
    self.auth=auth

  def connhive(self,database='fkdb',auth='LDAP'):
    conn = hive.Connection(host=self.host, port=self.port, username=self.user, auth=self.auth, password=self.pwd,
      database=self.database)
    logger.info(msg="hive server connect sucessfully {hivehost:%s,hiveport:%d,hiveuser:%s,hivepwd:%s,hiveauth:%s,hivedatabase:%s}"%(
      self.host,self.port,self.user,self.pwd,self.auth,self.database
    ))
    return conn

  def query_AllFileds_Dataframe(self,conn,table,param_dict,limit=None):
    base_table = self.database + '.' + table
    query_part='select * from %s where '%base_table
    params = []
    for key, value in param_dict.items():
      params.append(' '+key + '= \'%s\' ' % value + 'and')
    paramstemp = ''.join(params)
    params = paramstemp[:-3]
    if limit==None:
      hsql = query_part + params
    else:
      hsql=query_part+params+' limit %d'%limit
    logger.info(msg="exec hsql : %s"%hsql)
    # cursor=conn.cursor()
    # cursor.execute(hsql)
    # dataframe=pd.DataFrame(cursor.fetchall())
    dataframe = pd.read_sql(hsql, conn)
    return dataframe

  def query_selectFileds_Dataframe(self,conn,table,filed_list,param_dict,limit=None):
    base_table = self.database + '.' + table
    fileds=','.join(filed_list)
    query_part='select %s  from %s where '%(fileds,base_table)
    params = []
    for key, value in param_dict.items():
      params.append(' '+key + '= \'%s\' ' % value + 'and')
    paramstemp = ''.join(params)
    params = paramstemp[:-3]
    if limit==None:
      hsql = query_part + params
    else:
      hsql=query_part+params+ ' limit %d'%limit
    logger.info(msg="exec hsql : %s" % hsql)
    cursor = conn.cursor()
    cursor.execute(hsql)
    #dataframe=pd.DataFrame(cursor.fetchall())
    dataframe=pd.read_sql(hsql,conn,columns=filed_list)
    return dataframe



  def drop_partition(self,cursor,table,partition_param):
    base_table=self.database+'.'+table
    hsql_part='alter table %s drop if  exists  partition ( '%base_table
    params=[]
    for key, value in partition_param.items():
      params.append(key+'= \'%s\''%value+',')
    paramstemp=''.join(params)
    params=paramstemp[:-1]
    hsql=hsql_part+params+');'
    logger.info(msg="exec hsql : %s" % hsql)
    cursor.execute(hsql)

  def queryby_hsql_df(self,conn,hsql):
    logger.info(msg="exec hsql : %s" % hsql)
    cursor = conn.cursor()
    cursor.execute(hsql)
    # dataframe=pd.DataFrame(cursor.fetchall())
    dataframe = pd.read_sql(hsql, conn)
    return dataframe

logger = logging.getLogger(
    name=__name__,
)




from pyhive import hive
import pandas as  pd
import paramiko
import logging
import selectors
import socketserver
import threading
import time



class hive_client:

  def __init__(self,host,user,pwd,port=10000,database='fkdb',auth='LDAP'):
    self.host=host
    self.user=user
    self.pwd=pwd
    self.port=port
    self.database=database
    self.auth=auth

  def connhive(self,database='fkdb',auth='LDAP'):
    conn = hive.Connection(host=self.host, port=self.port, username=self.user, auth=self.auth, password=self.pwd,
      database=self.database)
    logger.info(msg="hive server connect sucessfully {hivehost:%s,hiveport:%d,hiveuser:%s,hivepwd:%s,hiveauth:%s,hivedatabase:%s}"%(
      self.host,self.port,self.user,self.pwd,self.auth,self.database
    ))
    return conn

  def query_AllFileds_Dataframe(self,conn,table,param_dict,limit=None):
    base_table = self.database + '.' + table
    query_part='select * from %s where '%base_table
    params = []
    for key, value in param_dict.items():
      params.append(' '+key + '= \'%s\' ' % value + 'and')
    paramstemp = ''.join(params)
    params = paramstemp[:-3]
    if limit==None:
      hsql = query_part + params
    else:
      hsql=query_part+params+' limit %d'%limit
    logger.info(msg="exec hsql : %s"%hsql)
    # cursor=conn.cursor()
    # cursor.execute(hsql)
    # dataframe=pd.DataFrame(cursor.fetchall())
    dataframe = pd.read_sql(hsql, conn)
    return dataframe

  def query_selectFileds_Dataframe(self,conn,table,filed_list,param_dict,limit=None):
    base_table = self.database + '.' + table
    fileds=','.join(filed_list)
    query_part='select %s  from %s where '%(fileds,base_table)
    params = []
    for key, value in param_dict.items():
      params.append(' '+key + '= \'%s\' ' % value + 'and')
    paramstemp = ''.join(params)
    params = paramstemp[:-3]
    if limit==None:
      hsql = query_part + params
    else:
      hsql=query_part+params+ ' limit %d'%limit
    logger.info(msg="exec hsql : %s" % hsql)
    cursor = conn.cursor()
    cursor.execute(hsql)
    #dataframe=pd.DataFrame(cursor.fetchall())
    dataframe=pd.read_sql(hsql,conn,columns=filed_list)
    return dataframe



  def drop_partition(self,cursor,table,partition_param):
    base_table=self.database+'.'+table
    hsql_part='alter table %s drop if  exists  partition ( '%base_table
    params=[]
    for key, value in partition_param.items():
      params.append(key+'= \'%s\''%value+',')
    paramstemp=''.join(params)
    params=paramstemp[:-1]
    hsql=hsql_part+params+');'
    logger.info(msg="exec hsql : %s" % hsql)
    cursor.execute(hsql)

  def queryby_hsql_df(self,conn,hsql):
    logger.info(msg="exec hsql : %s" % hsql)
    cursor = conn.cursor()
    cursor.execute(hsql)
    # dataframe=pd.DataFrame(cursor.fetchall())
    dataframe = pd.read_sql(hsql, conn)
    return dataframe

logger = logging.getLogger(
    name=__name__,
)


class SSH_Tunnel(
    socketserver.ThreadingTCPServer,
):
    daemon_threads = True
    allow_reuse_address = True

    def __init__(
        self,
        paramiko_session,
        remote_host,
        remote_port,
        bind_address_and_port=(
            '',
            0,
        ),
    ):
        self.paramiko_session = paramiko_session
        self.remote_host = remote_host
        self.remote_port = remote_port

        super().__init__(
            server_address=bind_address_and_port,
            RequestHandlerClass=SSHForwardingHandler,
            bind_and_activate=True,
        )
        self.bind_address, self.bind_port = self.server_address
        self.server_thread = threading.Thread(
            target=self.serve_forever,
            daemon=True,
        )

    def __str__(
        self,
    ):
        return 'Tunnel to \'{remote_host}:{remote_port}\', bound at \'{bind_address}:{bind_port}\'.'.format(
            remote_host=self.remote_host,
            remote_port=self.remote_port,
            bind_address=self.bind_address,
            bind_port=self.bind_port,
        )

    def start(
        self,
    ):
        self.server_thread.start()

    def __enter__(
        self,
    ):
        self.start()

        return self

    def __exit__(
        self,
        exc_type,
        exc_val,
        exc_tb,
    ):
        self.shutdown()


class SSHForwardingHandler(
    socketserver.BaseRequestHandler,
):
    buffer_size = 1024

    def __init__(
        self,
        request,
        client_address,
        server,
    ):
        self.selector = selectors.DefaultSelector()
        self.ssh_channel = None
        super().__init__(request, client_address, server)

    def _read_from_client(
        self,
        socket_obj,
        mask,
    ):
        self._transfer_data(
            src_socket=socket_obj,
            dest_socket=self.ssh_channel,
        )

    def _read_from_channel(
        self,
        socket_obj,
        mask,
    ):
        self._transfer_data(
            src_socket=socket_obj,
            dest_socket=self.request,
        )

    def _transfer_data(
        self,
        src_socket,
        dest_socket,
    ):
        src_socket.setblocking(
            False,
        )
        data = src_socket.recv(
            self.buffer_size,
        )
        if len(data):
            try:
                dest_socket.send(
                    data,
                )
            except BrokenPipeError:
                self.finish()

    def handle(
        self,
    ):
        peer_name = self.request.getpeername()
        try:
            self.ssh_channel = self.server.paramiko_session.get_transport().open_channel(
                kind='direct-tcpip',
                dest_addr=(
                    self.server.remote_host,
                    self.server.remote_port,
                ),
                src_addr=peer_name,
            )
        except Exception as error:
            logger.error(
                msg='Incoming request to {host}:{port} failed.'.format(
                    host=self.server.remote_host,
                    port=self.server.remote_port,
                ),
            )

            raise paramiko.SSHException(
                error,
            )
        else:
            self.selector.register(
                fileobj=self.ssh_channel,
                events=selectors.EVENT_READ,
                data=self._read_from_channel,
            )
            self.selector.register(
                fileobj=self.request,
                events=selectors.EVENT_READ,
                data=self._read_from_client,
            )
            if self.ssh_channel is None:
                logger.warning(
                    msg='Incoming request to {host}:{port} was rejected by the SSH server.'.format(
                        host=self.server.remote_host,
                        port=self.server.remote_port,
                    ),
                )

                self.finish()

            while True:
                events = self.selector.select()
                for key, mask in events:
                    callback = key.data
                    callback(
                        socket_obj=key.fileobj,
                        mask=mask,
                    )
                    if self.server._BaseServer__is_shut_down.is_set():
                        self.finish()
                time.sleep(0)

    def finish(
        self,
    ):
      
        if self.ssh_channel is not None:
            try:
              self.ssh_channel.shutdown(
                how=2,
              )
              self.ssh_channel.close()
            except Exception :
              self.request.shutdown(2,)
              self.request.close()
      # except Exception :
      #   logger.error(" maybe socket has something wrong but maybe you can get data result ,it's just work")
      # finally:
      #   logger.info("do next program")

class Jump_Tunnel_HIVE:
  logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
  logger = logging.getLogger(__name__)
  def __init__(self,jumphost,jumpport,jumpuser,jumppwd,tunnelhost,tunnelAPPport,localhost,localbindport,hiveusername, hivepassword):
    self.jumphost=jumphost
    self.jumpport=jumpport
    self.jumpuser=jumpuser
    self.jumppwd=jumppwd
    self.tunnelhost=tunnelhost
    self.tunnelAPPport=tunnelAPPport
    self.localhost=localhost
    self.localBindport=localbindport
    self.hiveusername=hiveusername
    self.hivepassword=hivepassword
  def  jump_connect(self):
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(
      paramiko.AutoAddPolicy(),
    )
    client.connect(
      hostname=self.jumphost,
      port=self.jumpport,
      username=self.jumpuser,
      password=self.jumppwd,
    )
    logger.info(msg="jump server info :{host: %s, port: %d,username: %s,password: %s}"%(self.jumphost,self.jumpport,self.jumpuser,self.jumppwd))

    return client
  def  jump_con_tunnel(self):
    client=self.jump_connect()
    tunnel=SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost,self.localBindport),
    )
    logger.info(msg="the ssh tunnel  for jump server %s" %tunnel)
    return tunnel
  # 获取hive数据生成 pandas dataframe ，
  # 参数 ：表名  分区参数字典  要查询的字段名列表，查询数量 默认  ，查询数据库 默认  登录hive方式 默认
  # pip install paramiko paramiko_tunnel  pyhive
  def get_JUMP_df(self,table, partions_param_dict, query_fileds_list, query_limit=100, database='fkdb', auth='LDAP'):
    tun=self.jump_con_tunnel()
    with tun:
       hive_cli = hive_client(self.localhost, self.hiveusername,self.hivepassword,self.localBindport)
       conn = hive_cli.connhive(database, auth)
       pd_dataframe = hive_cli.query_selectFileds_Dataframe(conn, table, query_fileds_list, partions_param_dict, query_limit)
       return pd_dataframe


  def get_JumpTunnel_df(self,table, partions_param_dict, query_fileds_list, query_limit=100, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
      paramiko_session=client,
      remote_host=self.tunnelhost,
      remote_port=self.tunnelAPPport,
      bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel :
       logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
       hive_cli = hive_client(self.localhost, self.hiveusername,self.hivepassword,self.localBindport)
       conn = hive_cli.connhive(database, auth)
       pd_dataframe = hive_cli.query_selectFileds_Dataframe(conn, table, query_fileds_list, partions_param_dict, query_limit)
       return pd_dataframe
  def get_JumpTunnel_hsql_df(self,hsql, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      pd_dataframe=hive_cli.queryby_hsql_df(conn,hsql)
      return  pd_dataframe

  def get_JumpTunnel_table_partitions_df(self,table,param_dict,limit=None, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      pd_dataframe = hive_cli.query_AllFileds_Dataframe(conn,table,param_dict,limit)
      return pd_dataframe

  def alter_table_Tunnel_by_hsql(self,hsql, database='fkdb', auth='LDAP'):
    client = self.jump_connect()
    with  SSH_Tunnel(
            paramiko_session=client,
            remote_host=self.tunnelhost,
            remote_port=self.tunnelAPPport,
            bind_address_and_port=(self.localhost, self.localBindport),
    ) as tunnel:
      logger.info(msg="the ssh tunnel  for jump server %s" % tunnel)
      hive_cli = hive_client(self.localhost, self.hiveusername, self.hivepassword, self.localBindport)
      conn = hive_cli.connhive(database, auth)
      cursor=conn.cursor()
      try:
        cursor.execute(hsql)
        logger.info(msg="hive cursor exec  successfully for : [ %s ]"%hsql)
      except:
        logger.error(msg="hive cursor  exec failed for [ %s ]"%hsql)


import paramiko
import logging
import selectors
import socketserver
import threading
import time

logger = logging.getLogger(
    name=__name__,
)


class SSH_Tunnel(
  socketserver.ThreadingTCPServer,
):
  logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
  logger = logging.getLogger(__name__)
  daemon_threads = True
  allow_reuse_address = True

  def __init__(
          self,
          paramiko_session,
          remote_host,
          remote_port,
          bind_address_and_port=(
                  '',
                  0,
          ),
  ):
    self.paramiko_session = paramiko_session
    self.remote_host = remote_host
    self.remote_port = remote_port

    super().__init__(
      server_address=bind_address_and_port,
      RequestHandlerClass=SSHForwardingHandler,
      bind_and_activate=True,
    )
    self.bind_address, self.bind_port = self.server_address
    self.server_thread = threading.Thread(
      target=self.serve_forever,
      daemon=True,
    )

  def __str__(
          self,
  ):
    return 'Tunnel to \'{remote_host}:{remote_port}\', bound at \'{bind_address}:{bind_port}\'.'.format(
      remote_host=self.remote_host,
      remote_port=self.remote_port,
      bind_address=self.bind_address,
      bind_port=self.bind_port,
    )

  def start(
          self,
  ):
    self.server_thread.start()

  def __enter__(
          self,
  ):
    self.start()

    return self

  def __exit__(
          self,
          exc_type,
          exc_val,
          exc_tb,
  ):
    self.shutdown()


class SSHForwardingHandler(
  socketserver.BaseRequestHandler,
):
  buffer_size = 1024

  def __init__(
          self,
          request,
          client_address,
          server,
  ):
    self.selector = selectors.DefaultSelector()
    self.ssh_channel = None
    super().__init__(request, client_address, server)

  def _read_from_client(
          self,
          socket_obj,
          mask,
  ):
    self._transfer_data(
      src_socket=socket_obj,
      dest_socket=self.ssh_channel,
    )

  def _read_from_channel(
          self,
          socket_obj,
          mask,
  ):
    self._transfer_data(
      src_socket=socket_obj,
      dest_socket=self.request,
    )

  def _transfer_data(
          self,
          src_socket,
          dest_socket,
  ):
    src_socket.setblocking(
      False,
    )
    data = src_socket.recv(
      self.buffer_size,
    )
    if len(data):
      try:
        dest_socket.send(
          data,
        )
      except BrokenPipeError:
        self.finish()

  def handle(
          self,
  ):
    peer_name = self.request.getpeername()
    try:
      self.ssh_channel = self.server.paramiko_session.get_transport().open_channel(
        kind='direct-tcpip',
        dest_addr=(
          self.server.remote_host,
          self.server.remote_port,
        ),
        src_addr=peer_name,
      )
    except Exception as error:
      logger.error(
        msg='Incoming request to {host}:{port} failed.'.format(
          host=self.server.remote_host,
          port=self.server.remote_port,
        ),
      )

      raise paramiko.SSHException(
        error,
      )
    else:
      self.selector.register(
        fileobj=self.ssh_channel,
        events=selectors.EVENT_READ,
        data=self._read_from_channel,
      )
      self.selector.register(
        fileobj=self.request,
        events=selectors.EVENT_READ,
        data=self._read_from_client,
      )
      if self.ssh_channel is None:
        logger.warning(
          msg='Incoming request to {host}:{port} was rejected by the SSH server.'.format(
            host=self.server.remote_host,
            port=self.server.remote_port,
          ),
        )

        self.finish()

      while True:
        events = self.selector.select()
        for key, mask in events:
          callback = key.data
          callback(
            socket_obj=key.fileobj,
            mask=mask,
          )
          if self.server._BaseServer__is_shut_down.is_set():
            self.finish()
        time.sleep(0)

  def finish(
          self,
  ):

    if self.ssh_channel is not None:
      try:
        self.ssh_channel.shutdown(
          how=2,
        )
        self.ssh_channel.close()
      except Exception:
        self.request.shutdown(2, )
        self.request.close()
  # except Exception :
  #   logger.error(" maybe socket has something wrong but maybe you can get data result ,it's just work")
  # finally:
  #   logger.info("do next program")




import logging
import sys
import os

if sys.version > "3":
    PY2 = False
else:
    PY2 = True


__version__ = '0.3.6'
__all__ = ['hive_client','Jump_Tunnel', 'ftps_client','Jump_Tunnel_HIVE', 'SSH_Tunnel']


# Initialize logger.
logger = logging.getLogger("SSH-Jump-Hive")
logger.setLevel(logging.INFO)
console_hdlr = logging.StreamHandler()
console_hdlr.setLevel(logging.INFO)
formatter = logging.Formatter("%(name)s   %(levelname)-8s %(message)s")
console_hdlr.setFormatter(formatter)
logger.addHandler(console_hdlr)



import codecs
import os

from setuptools import setup, find_packages

from jumps import __version__ as version
# See this web page for explanations:
# https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
PACKAGES = ["jumps",]
KEYWORDS = ["hive", "ssh-tunnel", "hfds", "machine learning"]
CLASSIFIERS = [
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "License :: OSI Approved :: Apache Software License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Scientific/Engineering",
]
# Project root
ROOT = os.path.abspath(os.path.dirname(__file__))

setup(
    name="ssh-jump_hive",
    description="ssh_jump_hive is a tools could  jump the jump machine  to connect hive get hive data to pandas dataframe",
    license="Apache 2.0",
    url='https://github.com/mullerhai/sshjumphive',
    version=version,
    author="mullerhai",
    author_email="hai710459649@foxmail.com",
    maintainer="muller helen",
    maintainer_email="hai710459649@foxmail.com",
    long_description=open('README.rst').read(),
    keywords=KEYWORDS,
    packages=find_packages(),
    classifiers=CLASSIFIERS,
    zip_safe=False,
    platforms=["all"],
    include_package_data=True,
    install_requires=[
        'pandas>=0.20.3',
        'PyHive>=0.5.1',
        'paramiko>=2.4.1',
        'selectors>=0.0.14',
        'sasl>=0.2.1',
        'thrift>=0.11.0',
        'thrift-sasl>=0.3.0',
        'hdfs>=2.1.0',
        'sklearn-pandas>=1.6.0',
        'scikit-learn>=0.19.1',
        'click>=6.7',

    ],
    entry_points={
        'console_scripts': [
            'jumps = jumps.jump_terminal:main',
            'jumpgui = jumps.jump_gui:main'

        ]
    },
)


